// Copyright 2026 Google LLC.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Code generated file. DO NOT EDIT.

// Package firebaseappdistribution provides access to the Firebase App Distribution API.
//
// For product documentation, see: https://firebase.google.com/products/app-distribution
//
// # Library status
//
// These client libraries are officially supported by Google. However, this
// library is considered complete and is in maintenance mode. This means
// that we will address critical bugs and security issues but will not add
// any new features.
//
// When possible, we recommend using our newer
// [Cloud Client Libraries for Go](https://pkg.go.dev/cloud.google.com/go)
// that are still actively being worked and iterated on.
//
// # Creating a client
//
// Usage example:
//
//	import "google.golang.org/api/firebaseappdistribution/v1alpha"
//	...
//	ctx := context.Background()
//	firebaseappdistributionService, err := firebaseappdistribution.NewService(ctx)
//
// In this example, Google Application Default Credentials are used for
// authentication. For information on how to create and obtain Application
// Default Credentials, see https://developers.google.com/identity/protocols/application-default-credentials.
//
// # Other authentication options
//
// To use an API key for authentication (note: some APIs do not support API
// keys), use [google.golang.org/api/option.WithAPIKey]:
//
//	firebaseappdistributionService, err := firebaseappdistribution.NewService(ctx, option.WithAPIKey("AIza..."))
//
// To use an OAuth token (e.g., a user token obtained via a three-legged OAuth
// flow, use [google.golang.org/api/option.WithTokenSource]:
//
//	config := &oauth2.Config{...}
//	// ...
//	token, err := config.Exchange(ctx, ...)
//	firebaseappdistributionService, err := firebaseappdistribution.NewService(ctx, option.WithTokenSource(config.TokenSource(ctx, token)))
//
// See [google.golang.org/api/option.ClientOption] for details on options.
package firebaseappdistribution // import "google.golang.org/api/firebaseappdistribution/v1alpha"

import (
	"bytes"
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"io"
	"log/slog"
	"net/http"
	"net/url"
	"strconv"
	"strings"

	"github.com/googleapis/gax-go/v2/internallog"
	googleapi "google.golang.org/api/googleapi"
	internal "google.golang.org/api/internal"
	gensupport "google.golang.org/api/internal/gensupport"
	option "google.golang.org/api/option"
	internaloption "google.golang.org/api/option/internaloption"
	htransport "google.golang.org/api/transport/http"
)

// Always reference these packages, just in case the auto-generated code
// below doesn't.
var _ = bytes.NewBuffer
var _ = strconv.Itoa
var _ = fmt.Sprintf
var _ = json.NewDecoder
var _ = io.Copy
var _ = url.Parse
var _ = gensupport.MarshalJSON
var _ = googleapi.Version
var _ = errors.New
var _ = strings.Replace
var _ = context.Canceled
var _ = internaloption.WithDefaultEndpoint
var _ = internal.Version
var _ = internallog.New

const apiId = "firebaseappdistribution:v1alpha"
const apiName = "firebaseappdistribution"
const apiVersion = "v1alpha"
const basePath = "https://firebaseappdistribution.googleapis.com/"
const basePathTemplate = "https://firebaseappdistribution.UNIVERSE_DOMAIN/"
const mtlsBasePath = "https://firebaseappdistribution.mtls.googleapis.com/"

// OAuth2 scopes used by this API.
const (
	// See, edit, configure, and delete your Google Cloud data and see the email
	// address for your Google Account.
	CloudPlatformScope = "https://www.googleapis.com/auth/cloud-platform"
)

// NewService creates a new Service.
func NewService(ctx context.Context, opts ...option.ClientOption) (*Service, error) {
	scopesOption := internaloption.WithDefaultScopes(
		"https://www.googleapis.com/auth/cloud-platform",
	)
	// NOTE: prepend, so we don't override user-specified scopes.
	opts = append([]option.ClientOption{scopesOption}, opts...)
	opts = append(opts, internaloption.WithDefaultEndpoint(basePath))
	opts = append(opts, internaloption.WithDefaultEndpointTemplate(basePathTemplate))
	opts = append(opts, internaloption.WithDefaultMTLSEndpoint(mtlsBasePath))
	opts = append(opts, internaloption.EnableNewAuthLibrary())
	client, endpoint, err := htransport.NewClient(ctx, opts...)
	if err != nil {
		return nil, err
	}
	s := &Service{client: client, BasePath: basePath, logger: internaloption.GetLogger(opts)}
	s.Apps = NewAppsService(s)
	s.Projects = NewProjectsService(s)
	if endpoint != "" {
		s.BasePath = endpoint
	}
	return s, nil
}

// New creates a new Service. It uses the provided http.Client for requests.
//
// Deprecated: please use NewService instead.
// To provide a custom HTTP client, use option.WithHTTPClient.
// If you are using google.golang.org/api/googleapis/transport.APIKey, use option.WithAPIKey with NewService instead.
func New(client *http.Client) (*Service, error) {
	if client == nil {
		return nil, errors.New("client is nil")
	}
	return NewService(context.TODO(), option.WithHTTPClient(client))
}

type Service struct {
	client    *http.Client
	logger    *slog.Logger
	BasePath  string // API endpoint base URL
	UserAgent string // optional additional User-Agent fragment

	Apps *AppsService

	Projects *ProjectsService
}

func (s *Service) userAgent() string {
	if s.UserAgent == "" {
		return googleapi.UserAgent
	}
	return googleapi.UserAgent + " " + s.UserAgent
}

func NewAppsService(s *Service) *AppsService {
	rs := &AppsService{s: s}
	rs.ReleaseByHash = NewAppsReleaseByHashService(s)
	rs.Releases = NewAppsReleasesService(s)
	rs.Testers = NewAppsTestersService(s)
	rs.UploadStatus = NewAppsUploadStatusService(s)
	return rs
}

type AppsService struct {
	s *Service

	ReleaseByHash *AppsReleaseByHashService

	Releases *AppsReleasesService

	Testers *AppsTestersService

	UploadStatus *AppsUploadStatusService
}

func NewAppsReleaseByHashService(s *Service) *AppsReleaseByHashService {
	rs := &AppsReleaseByHashService{s: s}
	return rs
}

type AppsReleaseByHashService struct {
	s *Service
}

func NewAppsReleasesService(s *Service) *AppsReleasesService {
	rs := &AppsReleasesService{s: s}
	rs.Notes = NewAppsReleasesNotesService(s)
	return rs
}

type AppsReleasesService struct {
	s *Service

	Notes *AppsReleasesNotesService
}

func NewAppsReleasesNotesService(s *Service) *AppsReleasesNotesService {
	rs := &AppsReleasesNotesService{s: s}
	return rs
}

type AppsReleasesNotesService struct {
	s *Service
}

func NewAppsTestersService(s *Service) *AppsTestersService {
	rs := &AppsTestersService{s: s}
	return rs
}

type AppsTestersService struct {
	s *Service
}

func NewAppsUploadStatusService(s *Service) *AppsUploadStatusService {
	rs := &AppsUploadStatusService{s: s}
	return rs
}

type AppsUploadStatusService struct {
	s *Service
}

func NewProjectsService(s *Service) *ProjectsService {
	rs := &ProjectsService{s: s}
	rs.Apps = NewProjectsAppsService(s)
	rs.Testers = NewProjectsTestersService(s)
	return rs
}

type ProjectsService struct {
	s *Service

	Apps *ProjectsAppsService

	Testers *ProjectsTestersService
}

func NewProjectsAppsService(s *Service) *ProjectsAppsService {
	rs := &ProjectsAppsService{s: s}
	rs.Releases = NewProjectsAppsReleasesService(s)
	rs.TestCases = NewProjectsAppsTestCasesService(s)
	return rs
}

type ProjectsAppsService struct {
	s *Service

	Releases *ProjectsAppsReleasesService

	TestCases *ProjectsAppsTestCasesService
}

func NewProjectsAppsReleasesService(s *Service) *ProjectsAppsReleasesService {
	rs := &ProjectsAppsReleasesService{s: s}
	rs.Tests = NewProjectsAppsReleasesTestsService(s)
	return rs
}

type ProjectsAppsReleasesService struct {
	s *Service

	Tests *ProjectsAppsReleasesTestsService
}

func NewProjectsAppsReleasesTestsService(s *Service) *ProjectsAppsReleasesTestsService {
	rs := &ProjectsAppsReleasesTestsService{s: s}
	return rs
}

type ProjectsAppsReleasesTestsService struct {
	s *Service
}

func NewProjectsAppsTestCasesService(s *Service) *ProjectsAppsTestCasesService {
	rs := &ProjectsAppsTestCasesService{s: s}
	return rs
}

type ProjectsAppsTestCasesService struct {
	s *Service
}

func NewProjectsTestersService(s *Service) *ProjectsTestersService {
	rs := &ProjectsTestersService{s: s}
	return rs
}

type ProjectsTestersService struct {
	s *Service
}

// AndroidxCrawlerOutputPoint: Point for describing bounding boxes tap
// locations Top left is 0,0
type AndroidxCrawlerOutputPoint struct {
	XCoordinate int64 `json:"xCoordinate,omitempty"`
	YCoordinate int64 `json:"yCoordinate,omitempty"`
	// ForceSendFields is a list of field names (e.g. "XCoordinate") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "XCoordinate") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s AndroidxCrawlerOutputPoint) MarshalJSON() ([]byte, error) {
	type NoMethod AndroidxCrawlerOutputPoint
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// AndroidxCrawlerOutputRectangle: Rectangle for describing bounding boxes
type AndroidxCrawlerOutputRectangle struct {
	Bottom int64 `json:"bottom,omitempty"`
	Left   int64 `json:"left,omitempty"`
	Right  int64 `json:"right,omitempty"`
	Top    int64 `json:"top,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Bottom") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Bottom") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s AndroidxCrawlerOutputRectangle) MarshalJSON() ([]byte, error) {
	type NoMethod AndroidxCrawlerOutputRectangle
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1Release: A release of a Firebase app.
type GoogleFirebaseAppdistroV1Release struct {
	// BinaryDownloadUri: Output only. A signed link (which expires in one hour) to
	// directly download the app binary (IPA/APK/AAB) file.
	BinaryDownloadUri string `json:"binaryDownloadUri,omitempty"`
	// BuildVersion: Output only. Build version of the release. For an Android
	// release, the build version is the `versionCode`. For an iOS release, the
	// build version is the `CFBundleVersion`.
	BuildVersion string `json:"buildVersion,omitempty"`
	// CreateTime: Output only. The time the release was created.
	CreateTime string `json:"createTime,omitempty"`
	// DisplayVersion: Output only. Display version of the release. For an Android
	// release, the display version is the `versionName`. For an iOS release, the
	// display version is the `CFBundleShortVersionString`.
	DisplayVersion string `json:"displayVersion,omitempty"`
	// ExpireTime: Output only. The time the release will expire.
	ExpireTime string `json:"expireTime,omitempty"`
	// FirebaseConsoleUri: Output only. A link to the Firebase console displaying a
	// single release.
	FirebaseConsoleUri string `json:"firebaseConsoleUri,omitempty"`
	// Name: The name of the release resource. Format:
	// `projects/{project_number}/apps/{app}/releases/{release}`
	Name string `json:"name,omitempty"`
	// ReleaseNotes: Notes of the release.
	ReleaseNotes *GoogleFirebaseAppdistroV1ReleaseNotes `json:"releaseNotes,omitempty"`
	// TestingUri: Output only. A link to the release in the tester web clip or
	// Android app that lets testers (which were granted access to the app) view
	// release notes and install the app onto their devices.
	TestingUri string `json:"testingUri,omitempty"`
	// UpdateTime: Output only. The time the release was last updated.
	UpdateTime string `json:"updateTime,omitempty"`
	// ForceSendFields is a list of field names (e.g. "BinaryDownloadUri") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "BinaryDownloadUri") to include in
	// API requests with the JSON null value. By default, fields with empty values
	// are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1Release) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1Release
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1ReleaseNotes: Notes that belong to a release.
type GoogleFirebaseAppdistroV1ReleaseNotes struct {
	// Text: The text of the release notes.
	Text string `json:"text,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Text") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Text") to include in API requests
	// with the JSON null value. By default, fields with empty values are omitted
	// from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1ReleaseNotes) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1ReleaseNotes
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1UploadReleaseMetadata: Operation metadata for
// `UploadRelease`.
type GoogleFirebaseAppdistroV1UploadReleaseMetadata struct {
}

// GoogleFirebaseAppdistroV1UploadReleaseResponse: Response message for
// `UploadRelease`.
type GoogleFirebaseAppdistroV1UploadReleaseResponse struct {
	// Release: Release associated with the uploaded binary.
	Release *GoogleFirebaseAppdistroV1Release `json:"release,omitempty"`
	// Result: Result of upload release.
	//
	// Possible values:
	//   "UPLOAD_RELEASE_RESULT_UNSPECIFIED" - Upload binary result unspecified
	//   "RELEASE_CREATED" - Upload binary resulted in a new release
	//   "RELEASE_UPDATED" - Upload binary updated an existing release
	//   "RELEASE_UNMODIFIED" - Upload binary resulted in a no-op. A release with
	// the exact same binary already exists.
	Result string `json:"result,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Release") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Release") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1UploadReleaseResponse) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1UploadReleaseResponse
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaAabCertificate: App bundle test certificate
type GoogleFirebaseAppdistroV1alphaAabCertificate struct {
	// CertificateHashMd5: MD5 hash of the certificate used to resign the AAB
	CertificateHashMd5 string `json:"certificateHashMd5,omitempty"`
	// CertificateHashSha1: SHA1 hash of the certificate used to resign the AAB
	CertificateHashSha1 string `json:"certificateHashSha1,omitempty"`
	// CertificateHashSha256: SHA256 hash of the certificate used to resign the AAB
	CertificateHashSha256 string `json:"certificateHashSha256,omitempty"`
	// ForceSendFields is a list of field names (e.g. "CertificateHashMd5") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "CertificateHashMd5") to include
	// in API requests with the JSON null value. By default, fields with empty
	// values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaAabCertificate) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaAabCertificate
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

type GoogleFirebaseAppdistroV1alphaAiInstructions struct {
	// Steps: Required. Steps to be accomplished by the AI
	Steps []*GoogleFirebaseAppdistroV1alphaAiStep `json:"steps,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Steps") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Steps") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaAiInstructions) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaAiInstructions
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaAiStep: A step to be accomplished by the AI
type GoogleFirebaseAppdistroV1alphaAiStep struct {
	// Assertion: An assertion to be checked by the AI
	Assertion string `json:"assertion,omitempty"`
	// Goal: A goal to be accomplished by the AI
	Goal string `json:"goal,omitempty"`
	// Hint: Optional. Hint text containing suggestions to help the agent
	// accomplish the goal
	Hint string `json:"hint,omitempty"`
	// SuccessCriteria: Optional. A visual description of the screen's expected
	// state after the step has been successfully completed. This is referred to as
	// the "final screen assertion" in the Firebase console and CLI tools. This
	// field must be provided for the last step in a test case, and is optional for
	// all other steps.
	SuccessCriteria string `json:"successCriteria,omitempty"`
	// TestCase: Output only. The test case that contained this step. Note: The
	// test case may have changed or been deleted since this step was created.
	// Format: `projects/{project_number}/apps/{app}/testCases/{test_case}`
	TestCase string `json:"testCase,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Assertion") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Assertion") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaAiStep) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaAiStep
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaAiStepResult: Captures the results of an
// AiStep
type GoogleFirebaseAppdistroV1alphaAiStepResult struct {
	// AssertionDetails: Output only. Details for an assertion step.
	AssertionDetails *GoogleFirebaseAppdistroV1alphaAssertionDetails `json:"assertionDetails,omitempty"`
	// GoalDetails: Output only. Details for a goal step.
	GoalDetails *GoogleFirebaseAppdistroV1alphaGoalDetails `json:"goalDetails,omitempty"`
	// State: Output only. The current state of the step
	//
	// Possible values:
	//   "STEP_STATE_UNSPECIFIED" - Step state unspecified
	//   "IN_PROGRESS" - The step is in progress
	//   "PASSED" - The step has completed successfully
	//   "FAILED" - The step has failed
	//   "TIMED_OUT" - The test timed out during this step
	//   "GOAL_ACTION_LIMIT_REACHED" - The number of actions needed to reach the
	// goal exceeded its limit
	State string `json:"state,omitempty"`
	// Step: Required. The step performed by the AI
	Step *GoogleFirebaseAppdistroV1alphaAiStep `json:"step,omitempty"`
	// ForceSendFields is a list of field names (e.g. "AssertionDetails") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "AssertionDetails") to include in
	// API requests with the JSON null value. By default, fields with empty values
	// are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaAiStepResult) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaAiStepResult
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaApp: An app.
type GoogleFirebaseAppdistroV1alphaApp struct {
	// AabCertificate: App bundle test certificate generated for the app.
	AabCertificate *GoogleFirebaseAppdistroV1alphaAabCertificate `json:"aabCertificate,omitempty"`
	// AabState: App bundle state. Only valid for android apps. The app_view field
	// in the request must be set to FULL in order for this to be populated.
	//
	// Possible values:
	//   "AAB_STATE_UNSPECIFIED" - Aab state unspecified
	//   "ACTIVE" - App can receive app bundle uploads
	//   "PLAY_ACCOUNT_NOT_LINKED" - Firebase project is not linked to a Play
	// developer account
	//   "NO_APP_WITH_GIVEN_BUNDLE_ID_IN_PLAY_ACCOUNT" - There is no app in linked
	// Play developer account with the same bundle id
	//   "APP_NOT_PUBLISHED" - The app in Play developer account is not in a
	// published state
	//   "AAB_STATE_UNAVAILABLE" - Play App status is unavailable
	//   "PLAY_IAS_TERMS_NOT_ACCEPTED" - Play IAS terms not accepted
	AabState string `json:"aabState,omitempty"`
	// AppId: Firebase gmp app id
	AppId string `json:"appId,omitempty"`
	// BundleId: Bundle identifier
	BundleId string `json:"bundleId,omitempty"`
	// ContactEmail: Developer contact email for testers to reach out to about
	// privacy or support issues.
	ContactEmail string `json:"contactEmail,omitempty"`
	// Platform: iOS or Android
	Platform string `json:"platform,omitempty"`
	// ProjectNumber: Project number of the Firebase project, for example
	// 300830567303.
	ProjectNumber string `json:"projectNumber,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "AabCertificate") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "AabCertificate") to include in
	// API requests with the JSON null value. By default, fields with empty values
	// are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaApp) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaApp
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaAppCrash: An app crash that occurred during an
// automated test.
type GoogleFirebaseAppdistroV1alphaAppCrash struct {
	// Message: Output only. The message associated with the crash.
	Message string `json:"message,omitempty"`
	// StackTrace: Output only. The raw stack trace.
	StackTrace string `json:"stackTrace,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Message") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Message") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaAppCrash) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaAppCrash
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaAssertionDetails: Details for an assertion
// step.
type GoogleFirebaseAppdistroV1alphaAssertionDetails struct {
	// Explanation: Output only. An explanation justifying the assertion result.
	Explanation string `json:"explanation,omitempty"`
	// Result: Output only. The result of the assertion.
	Result bool `json:"result,omitempty"`
	// Screenshot: Output only. The screenshot used in the context of this
	// assertion.
	Screenshot *GoogleFirebaseAppdistroV1alphaScreenshot `json:"screenshot,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Explanation") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Explanation") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaAssertionDetails) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaAssertionDetails
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaBatchDeleteTestCasesRequest: The request
// message for `BatchDeleteTestCase`.
type GoogleFirebaseAppdistroV1alphaBatchDeleteTestCasesRequest struct {
	// Names: Required. The name of the test cases to delete. A maximum number of
	// 1000 test cases can be deleted in one batch Format:
	// `projects/{project_number}/apps/{app}/testCases/{test_case}`
	Names []string `json:"names,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Names") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Names") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaBatchDeleteTestCasesRequest) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaBatchDeleteTestCasesRequest
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesRequest: The request
// message for `BatchUpdateTestCase`.
type GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesRequest struct {
	// Requests: Required. The update requests. A maximum number of 1000 test cases
	// can be updated in one batch
	Requests []*GoogleFirebaseAppdistroV1alphaUpdateTestCaseRequest `json:"requests,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Requests") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Requests") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesRequest) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesRequest
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesResponse: The response
// message for `BatchUpdateTestCase`.
type GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesResponse struct {
	// TestCases: The updated test cases.
	TestCases []*GoogleFirebaseAppdistroV1alphaTestCase `json:"testCases,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "TestCases") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "TestCases") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesResponse) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesResponse
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaCancelReleaseTestResponse: The (empty)
// response message for `CancelReleaseTest`.
type GoogleFirebaseAppdistroV1alphaCancelReleaseTestResponse struct {
	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
}

// GoogleFirebaseAppdistroV1alphaClearTestCaseCacheRequest: Request message for
// `ClearTestCaseCache`.
type GoogleFirebaseAppdistroV1alphaClearTestCaseCacheRequest struct {
	// TestDevices: Optional. The list of devices for which to clear the cache. If
	// not present, clear all of them.
	TestDevices []*GoogleFirebaseAppdistroV1alphaTestDevice `json:"testDevices,omitempty"`
	// ForceSendFields is a list of field names (e.g. "TestDevices") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "TestDevices") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaClearTestCaseCacheRequest) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaClearTestCaseCacheRequest
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaClearTestCaseCacheResponse: Response empty
// (google.protobuf.Empty) message for `ClearTestCaseCache`
type GoogleFirebaseAppdistroV1alphaClearTestCaseCacheResponse struct {
	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
}

// GoogleFirebaseAppdistroV1alphaCreateReleaseNotesRequest: The request message
// for `CreateReleaseNotes`.
type GoogleFirebaseAppdistroV1alphaCreateReleaseNotesRequest struct {
	// ReleaseNotes: The actual release notes body from the user
	ReleaseNotes *GoogleFirebaseAppdistroV1alphaReleaseNotes `json:"releaseNotes,omitempty"`
	// ForceSendFields is a list of field names (e.g. "ReleaseNotes") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "ReleaseNotes") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaCreateReleaseNotesRequest) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaCreateReleaseNotesRequest
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaCreateReleaseNotesResponse: The response
// message for `CreateReleaseNotes`.
type GoogleFirebaseAppdistroV1alphaCreateReleaseNotesResponse struct {
	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
}

// GoogleFirebaseAppdistroV1alphaDeviceAction: A high level action taken by the
// AI on the device, potentially involving multiple taps, text entries, waits,
// etc.
type GoogleFirebaseAppdistroV1alphaDeviceAction struct {
	// Description: Output only. A short description of the high level action taken
	// by the AI agent.
	Description string `json:"description,omitempty"`
	// DeviceInteractions: Output only. The interactions made with the device as
	// part of this higher level action taken by the agent, such as taps, text
	// entries, waits, etc.
	DeviceInteractions []*GoogleFirebaseAppdistroV1alphaDeviceInteraction `json:"deviceInteractions,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Description") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Description") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaDeviceAction) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaDeviceAction
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaDeviceExecution: The results of running an
// automated test on a particular device.
type GoogleFirebaseAppdistroV1alphaDeviceExecution struct {
	// AiStepResults: Output only. Results of the AI steps if passed in
	AiStepResults []*GoogleFirebaseAppdistroV1alphaAiStepResult `json:"aiStepResults,omitempty"`
	// AppCrash: Output only. An app crash, if any occurred during the test.
	AppCrash *GoogleFirebaseAppdistroV1alphaAppCrash `json:"appCrash,omitempty"`
	// CrawlGraphUri: Output only. A URI to an image of the Robo crawl graph.
	CrawlGraphUri string `json:"crawlGraphUri,omitempty"`
	// Device: Required. The device that the test was run on.
	Device *GoogleFirebaseAppdistroV1alphaTestDevice `json:"device,omitempty"`
	// ExecutionType: Output only. The type of execution for the test.
	//
	// Possible values:
	//   "EXECUTION_TYPE_UNSPECIFIED" - Execution type unspecified.
	//   "AI" - An AI-driven test.
	//   "ACTION_BASED_REPLAY" - A replay of a previous test run.
	//   "AI_REPLAY" - A failed replay that was rerun using the AI agent.
	//   "RANDOM_CRAWL" - A random Robo test.
	ExecutionType string `json:"executionType,omitempty"`
	// FailedReason: Output only. The reason why the test failed.
	//
	// Possible values:
	//   "FAILED_REASON_UNSPECIFIED" - Reason unspecified.
	//   "CRASHED" - The app crashed during the test.
	//   "NOT_INSTALLED" - If an app is not installed and thus no test can be run
	// with the app. This might be caused by trying to run a test on an unsupported
	// platform.
	//   "UNABLE_TO_CRAWL" - If the app could not be crawled (possibly because the
	// app did not start).
	//   "DEVICE_OUT_OF_MEMORY" - If the device ran out of memory during the test.
	//   "FAILED_AI_STEP" - At least one AI step failed.
	//   "TIMED_OUT" - The crawl reached the time limit before the test could be
	// completed.
	FailedReason string `json:"failedReason,omitempty"`
	// FinalAiAssertionMissing: Output only. Indicates that the test replayed saved
	// actions and concluded without a final AI assertion.
	FinalAiAssertionMissing bool `json:"finalAiAssertionMissing,omitempty"`
	// InconclusiveReason: Output only. The reason why the test was inconclusive.
	//
	// Possible values:
	//   "INCONCLUSIVE_REASON_UNSPECIFIED" - Reason unspecified.
	//   "QUOTA_EXCEEDED" - Not enough quota remained to run the test.
	//   "INFRASTRUCTURE_FAILURE" - The outcome of the test could not be determined
	// because of a failure in the test running infrastructure.
	//   "SERVICE_NOT_ACTIVATED" - A required cloud service api is not activated
	// (Google Cloud Testing API or Cloud Tool Results API).
	//   "NO_SIGNATURE" - The app was not signed.
	//   "NO_LAUNCHER_ACTIVITY" - A main launcher activity could not be found.
	//   "FORBIDDEN_PERMISSIONS" - The app declares one or more permissions that
	// are not allowed.
	//   "DEVICE_ADMIN_RECEIVER" - Device administrator applications are not
	// allowed.
	//   "NO_CODE_APK" - APK contains no code. See also
	// https://developer.android.com/guide/topics/manifest/application-element.html#code
	//   "INVALID_APK_PREVIEW_SDK" - APK is built for a preview SDK which is
	// unsupported.
	InconclusiveReason string `json:"inconclusiveReason,omitempty"`
	// Name: Identifier. The name of the device execution resource. Format:
	// `projects/{project_number}/apps/{app}/releases/{release}/tests/{test}/deviceE
	// xecutions/{device_execution}`
	Name string `json:"name,omitempty"`
	// OriginDeviceExecution: Output only. The device execution from which cached
	// steps were used during this execution. Note: This field is only populated
	// for ACTION_BASED_REPLAY executions. If the original device execution no
	// longer exists, this field will be empty. Format:
	// `projects/{project_number}/apps/{app}/releases/{release}/tests/{test}/deviceE
	// xecutions/{device_execution}`
	OriginDeviceExecution string `json:"originDeviceExecution,omitempty"`
	// ResultsStoragePath: Output only. The path to a directory in Cloud Storage
	// that will eventually contain the results for this execution. For example,
	// gs://bucket/Nexus5-18-en-portrait.
	ResultsStoragePath string `json:"resultsStoragePath,omitempty"`
	// RoboStats: Output only. The statistics collected during the Robo test.
	RoboStats *GoogleFirebaseAppdistroV1alphaRoboStats `json:"roboStats,omitempty"`
	// ScreenshotUris: Output only. A list of screenshot image URIs taken from the
	// Robo crawl. The file names are numbered by the order in which they were
	// taken.
	ScreenshotUris []string `json:"screenshotUris,omitempty"`
	// State: Output only. The state of the test.
	//
	// Possible values:
	//   "TEST_STATE_UNSPECIFIED" - Test state unspecified.
	//   "IN_PROGRESS" - The test is in progress.
	//   "PASSED" - The test has passed.
	//   "FAILED" - The test has failed.
	//   "INCONCLUSIVE" - The test was inconclusive.
	State string `json:"state,omitempty"`
	// VideoStartTime: Output only. The time at which the video started recording.
	VideoStartTime string `json:"videoStartTime,omitempty"`
	// VideoUri: Output only. A URI to a video of the test run.
	VideoUri string `json:"videoUri,omitempty"`
	// ForceSendFields is a list of field names (e.g. "AiStepResults") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "AiStepResults") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaDeviceExecution) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaDeviceExecution
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaDeviceInteraction: An interaction with the
// device, such as a tap, text entry, wait, etc.
type GoogleFirebaseAppdistroV1alphaDeviceInteraction struct {
	// BackAction: Output only. A back action.
	BackAction *GoogleFirebaseAppdistroV1alphaDeviceInteractionBack `json:"backAction,omitempty"`
	// DragAndDrop: Output only. A drag and drop action.
	DragAndDrop *GoogleFirebaseAppdistroV1alphaDeviceInteractionDragAndDrop `json:"dragAndDrop,omitempty"`
	// EnterText: Output only. A text entry action, that enters text into a
	// particular text field, clearing any existing text in the field. Unlike
	// `text_input` this action does not require any other actions such as a tap to
	// be performed before it can enter the text.
	EnterText *GoogleFirebaseAppdistroV1alphaDeviceInteractionEnterText `json:"enterText,omitempty"`
	// KeyCode: Output only. Key code for a key event action.
	KeyCode string `json:"keyCode,omitempty"`
	// LongPress: Output only. A long press (tap and hold) action.
	LongPress *AndroidxCrawlerOutputPoint `json:"longPress,omitempty"`
	// Screenshot: Output only. The screenshot used in the context of this action.
	// The screen may have changed before the action was actually taken.
	Screenshot *GoogleFirebaseAppdistroV1alphaScreenshot `json:"screenshot,omitempty"`
	// Swipe: Output only. A swipe action.
	Swipe *GoogleFirebaseAppdistroV1alphaDeviceInteractionSwipe `json:"swipe,omitempty"`
	// Tap: Output only. A tap action.
	Tap *AndroidxCrawlerOutputPoint `json:"tap,omitempty"`
	// TextInput: Output only. A text input action, that types some text into
	// whatever field is currently focused, if any. Unlike `enter_text` this action
	// requires that the field be brought into focus first, for example by emitting
	// a tap action before this one.
	TextInput string `json:"textInput,omitempty"`
	// Wait: Output only. A wait action.
	Wait *GoogleFirebaseAppdistroV1alphaDeviceInteractionWait `json:"wait,omitempty"`
	// ForceSendFields is a list of field names (e.g. "BackAction") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "BackAction") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaDeviceInteraction) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaDeviceInteraction
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaDeviceInteractionBack: A back action.
type GoogleFirebaseAppdistroV1alphaDeviceInteractionBack struct {
}

// GoogleFirebaseAppdistroV1alphaDeviceInteractionDragAndDrop: A drag and drop
// action.
type GoogleFirebaseAppdistroV1alphaDeviceInteractionDragAndDrop struct {
	// End: Output only. The end point of the drag and drop.
	End *AndroidxCrawlerOutputPoint `json:"end,omitempty"`
	// Start: Output only. The start point of the drag and drop.
	Start *AndroidxCrawlerOutputPoint `json:"start,omitempty"`
	// ForceSendFields is a list of field names (e.g. "End") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "End") to include in API requests
	// with the JSON null value. By default, fields with empty values are omitted
	// from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaDeviceInteractionDragAndDrop) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaDeviceInteractionDragAndDrop
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaDeviceInteractionEnterText: A text entry
// action, that enters text into a particular text field, clearing any existing
// text in the field.
type GoogleFirebaseAppdistroV1alphaDeviceInteractionEnterText struct {
	// ElementBounds: Output only. The visible bounds of the element to enter text
	// into.
	ElementBounds *AndroidxCrawlerOutputRectangle `json:"elementBounds,omitempty"`
	// Text: Output only. The text to enter.
	Text string `json:"text,omitempty"`
	// ForceSendFields is a list of field names (e.g. "ElementBounds") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "ElementBounds") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaDeviceInteractionEnterText) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaDeviceInteractionEnterText
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaDeviceInteractionSwipe: A swipe action.
type GoogleFirebaseAppdistroV1alphaDeviceInteractionSwipe struct {
	// End: Output only. The end point of the swipe.
	End *AndroidxCrawlerOutputPoint `json:"end,omitempty"`
	// Start: Output only. The start point of the swipe.
	Start *AndroidxCrawlerOutputPoint `json:"start,omitempty"`
	// ForceSendFields is a list of field names (e.g. "End") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "End") to include in API requests
	// with the JSON null value. By default, fields with empty values are omitted
	// from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaDeviceInteractionSwipe) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaDeviceInteractionSwipe
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaDeviceInteractionWait: A wait action.
type GoogleFirebaseAppdistroV1alphaDeviceInteractionWait struct {
	// Duration: Output only. The duration of the wait.
	Duration string `json:"duration,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Duration") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Duration") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaDeviceInteractionWait) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaDeviceInteractionWait
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseRequest: The request
// message for `EnableAccessOnRelease`.
type GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseRequest struct {
	// BuildVersion: Optional. Ignored. Used to be build version of the app release
	// if an instance identifier was provided for the release_id.
	BuildVersion string `json:"buildVersion,omitempty"`
	// DisplayVersion: Optional. Ignored. Used to be display version of the app
	// release if an instance identifier was provided for the release_id.
	DisplayVersion string `json:"displayVersion,omitempty"`
	// Emails: Optional. An email address which should get access to this release,
	// for example rebeccahe@google.com
	Emails []string `json:"emails,omitempty"`
	// GroupIds: Optional. A repeated list of group aliases to enable access to a
	// release for Note: This field is misnamed, but can't be changed because we
	// need to maintain compatibility with old build tools
	GroupIds []string `json:"groupIds,omitempty"`
	// ForceSendFields is a list of field names (e.g. "BuildVersion") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "BuildVersion") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseRequest) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseRequest
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseResponse: The response
// message for `EnableAccessOnRelease`.
type GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseResponse struct {
	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
}

// GoogleFirebaseAppdistroV1alphaGetReleaseByUploadHashResponse: Response
// object to get the release given a upload hash
type GoogleFirebaseAppdistroV1alphaGetReleaseByUploadHashResponse struct {
	// Release: Release object
	Release *GoogleFirebaseAppdistroV1alphaRelease `json:"release,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "Release") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Release") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaGetReleaseByUploadHashResponse) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaGetReleaseByUploadHashResponse
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaGetTesterUdidsResponse: Response containing
// the UDIDs of tester iOS devices in a project
type GoogleFirebaseAppdistroV1alphaGetTesterUdidsResponse struct {
	// TesterUdids: The UDIDs of tester iOS devices in a project
	TesterUdids []*GoogleFirebaseAppdistroV1alphaTesterUdid `json:"testerUdids,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "TesterUdids") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "TesterUdids") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaGetTesterUdidsResponse) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaGetTesterUdidsResponse
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaGetUploadStatusResponse: The response message
// for `GetUploadStatus`.
type GoogleFirebaseAppdistroV1alphaGetUploadStatusResponse struct {
	// ErrorCode: The error code associated with (only set on "FAILURE")
	//
	// Possible values:
	//   "ERROR_UNSPECIFIED"
	//   "INVALID_ZIP"
	//   "MISSING_PLIST"
	//   "MISSING_PROFILE"
	//   "VERSION_TOO_LONG"
	//   "MISSING_UUIDS"
	//   "MISSING_RESOURCES"
	//   "MISSING_MANIFEST"
	//   "IOS_METADATA_ERROR"
	//   "ANDROID_METADATA_ERROR"
	//   "UNSUPPORTED_PLATFORM_TYPE"
	//   "BUNDLE_ID_MISMATCH"
	//   "APK_NOT_ZIP_ALIGNED"
	//   "INVALID_CERTIFICATE"
	//   "APK_TOO_LARGE"
	//   "AAB_NOT_PUBLISHED"
	//   "INVALID_PLIST_DEVICE_FAMILIES"
	//   "AAB_TOS_NOT_ACCEPTED"
	//   "APP_NAME_TOO_LONG"
	//   "AAB_DEVELOPER_ACCOUNT_NOT_LINKED"
	//   "AAB_NO_APP_WITH_GIVEN_PACKAGE_NAME_IN_ACCOUNT"
	//   "AAB_UPLOAD_ERROR"
	//   "APP_NOT_FOUND" - Happens if the Firebase app no longer exists by the time
	// of extraction
	ErrorCode string `json:"errorCode,omitempty"`
	// Message: Any additional context for the given upload status (e.g. error
	// message) Meant to be displayed to the client
	Message string `json:"message,omitempty"`
	// Release: The release that was created from the upload (only set on
	// "SUCCESS")
	Release *GoogleFirebaseAppdistroV1alphaRelease `json:"release,omitempty"`
	// Status: The status of the upload
	//
	// Possible values:
	//   "STATUS_UNSPECIFIED" - Status unspecified.
	//   "IN_PROGRESS" - The upload is in progress.
	//   "ALREADY_UPLOADED" - The binary has already been uploaded.
	//   "SUCCESS" - The upload was successful.
	//   "ERROR" - The upload failed.
	Status string `json:"status,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "ErrorCode") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "ErrorCode") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaGetUploadStatusResponse) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaGetUploadStatusResponse
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaGoalAction: An action taken by the AI agent
// while attempting to accomplish a goal.
type GoogleFirebaseAppdistroV1alphaGoalAction struct {
	// CachingType: Output only. The type of caching used to determine the action.
	//
	// Possible values:
	//   "CACHING_TYPE_UNSPECIFIED" - Caching type unspecified.
	//   "NO_CACHING" - The model alone was used to determine the action with no
	// caching.
	//   "CACHE_AND_MODEL" - The model used information from a previous successful
	// run to determine the action.
	//   "CACHE_ONLY" - The action was replayed from a previous successful run
	// without model input.
	CachingType string `json:"cachingType,omitempty"`
	// DebugInfo: Output only. Debug information explaining why the agent to the
	// specific action.
	DebugInfo *GoogleFirebaseAppdistroV1alphaGoalActionDebugInfo `json:"debugInfo,omitempty"`
	// DeviceAction: Output only. A high level action taken by the AI on the
	// device.
	DeviceAction *GoogleFirebaseAppdistroV1alphaDeviceAction `json:"deviceAction,omitempty"`
	// Explanation: Output only. An explanation justifying why the action was
	// taken.
	Explanation string `json:"explanation,omitempty"`
	// StartTime: Output only. The time at which the action started.
	StartTime string `json:"startTime,omitempty"`
	// TerminalAction: Output only. An action taken by the AI to end the goal.
	TerminalAction *GoogleFirebaseAppdistroV1alphaTerminalAction `json:"terminalAction,omitempty"`
	// ForceSendFields is a list of field names (e.g. "CachingType") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "CachingType") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaGoalAction) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaGoalAction
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaGoalActionDebugInfo: Information to help the
// customer understand why the agent took this action.
type GoogleFirebaseAppdistroV1alphaGoalActionDebugInfo struct {
	// AnnotatedScreenshotUri: Output only. URI of the screenshot with elements
	// labeled which was used by the agent.
	AnnotatedScreenshotUri string `json:"annotatedScreenshotUri,omitempty"`
	// JsonUri: Output only. Structured data explaining the agent's choice.
	JsonUri string `json:"jsonUri,omitempty"`
	// ForceSendFields is a list of field names (e.g. "AnnotatedScreenshotUri") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "AnnotatedScreenshotUri") to
	// include in API requests with the JSON null value. By default, fields with
	// empty values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaGoalActionDebugInfo) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaGoalActionDebugInfo
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaGoalDetails: Details for a goal step.
type GoogleFirebaseAppdistroV1alphaGoalDetails struct {
	// GoalActions: Output only. The actions taken by the AI while attempting to
	// accomplish the goal.
	GoalActions []*GoogleFirebaseAppdistroV1alphaGoalAction `json:"goalActions,omitempty"`
	// ForceSendFields is a list of field names (e.g. "GoalActions") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "GoalActions") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaGoalDetails) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaGoalDetails
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaJwt: A JWT token.
type GoogleFirebaseAppdistroV1alphaJwt struct {
	// Token: The JWT token (three Base64URL-encoded strings joined by dots).
	Token string `json:"token,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "Token") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Token") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaJwt) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaJwt
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaListReleaseTestsResponse: The response message
// for `ListReleaseTests`.
type GoogleFirebaseAppdistroV1alphaListReleaseTestsResponse struct {
	// NextPageToken: A short-lived token, which can be sent as `pageToken` to
	// retrieve the next page. If this field is omitted, there are no subsequent
	// pages.
	NextPageToken string `json:"nextPageToken,omitempty"`
	// ReleaseTests: The tests listed.
	ReleaseTests []*GoogleFirebaseAppdistroV1alphaReleaseTest `json:"releaseTests,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "NextPageToken") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "NextPageToken") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaListReleaseTestsResponse) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaListReleaseTestsResponse
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaListTestCasesResponse: The response message
// for `ListTestCases`.
type GoogleFirebaseAppdistroV1alphaListTestCasesResponse struct {
	// NextPageToken: A token, which can be sent as `page_token` to retrieve the
	// next page. If this field is omitted, there are no subsequent pages.
	NextPageToken string `json:"nextPageToken,omitempty"`
	// TestCases: The test cases from the specified app.
	TestCases []*GoogleFirebaseAppdistroV1alphaTestCase `json:"testCases,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "NextPageToken") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "NextPageToken") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaListTestCasesResponse) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaListTestCasesResponse
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaLoginCredential: Login credential for
// automated tests
type GoogleFirebaseAppdistroV1alphaLoginCredential struct {
	// FieldHints: Optional. Hints to the crawler for identifying input fields
	FieldHints *GoogleFirebaseAppdistroV1alphaLoginCredentialFieldHints `json:"fieldHints,omitempty"`
	// Google: Optional. Are these credentials for Google?
	Google bool `json:"google,omitempty"`
	// Password: Optional. Password for automated tests
	Password string `json:"password,omitempty"`
	// Username: Optional. Username for automated tests
	Username string `json:"username,omitempty"`
	// ForceSendFields is a list of field names (e.g. "FieldHints") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "FieldHints") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaLoginCredential) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaLoginCredential
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaLoginCredentialFieldHints: Hints to the
// crawler for identifying input fields
type GoogleFirebaseAppdistroV1alphaLoginCredentialFieldHints struct {
	// PasswordResourceName: Required. The Android resource name of the password UI
	// element. For example, in Java: R.string.foo in xml: @string/foo Only the
	// "foo" part is needed. Reference doc:
	// https://developer.android.com/guide/topics/resources/accessing-resources.html
	PasswordResourceName string `json:"passwordResourceName,omitempty"`
	// UsernameResourceName: Required. The Android resource name of the username UI
	// element. For example, in Java: R.string.foo in xml: @string/foo Only the
	// "foo" part is needed. Reference doc:
	// https://developer.android.com/guide/topics/resources/accessing-resources.html
	UsernameResourceName string `json:"usernameResourceName,omitempty"`
	// ForceSendFields is a list of field names (e.g. "PasswordResourceName") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "PasswordResourceName") to include
	// in API requests with the JSON null value. By default, fields with empty
	// values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaLoginCredentialFieldHints) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaLoginCredentialFieldHints
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaRelease: Proto defining a release object
type GoogleFirebaseAppdistroV1alphaRelease struct {
	// BuildVersion: Release build version
	BuildVersion string `json:"buildVersion,omitempty"`
	// DisplayVersion: Release version
	DisplayVersion string `json:"displayVersion,omitempty"`
	// DistributedAt: Timestamp when the release was created
	DistributedAt string `json:"distributedAt,omitempty"`
	// Id: Release Id
	Id string `json:"id,omitempty"`
	// InstanceId: Instance id of the release
	InstanceId string `json:"instanceId,omitempty"`
	// LastActivityAt: Last activity timestamp
	LastActivityAt string `json:"lastActivityAt,omitempty"`
	// OpenInvitationCount: Number of testers who have open invitations for the
	// release
	OpenInvitationCount int64 `json:"openInvitationCount,omitempty"`
	// ReceivedAt: unused.
	ReceivedAt string `json:"receivedAt,omitempty"`
	// ReleaseNotesSummary: Release notes summary
	ReleaseNotesSummary string `json:"releaseNotesSummary,omitempty"`
	// TesterCount: Count of testers added to the release
	TesterCount int64 `json:"testerCount,omitempty"`
	// TesterWithInstallCount: Number of testers who have installed the release
	TesterWithInstallCount int64 `json:"testerWithInstallCount,omitempty"`
	// ForceSendFields is a list of field names (e.g. "BuildVersion") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "BuildVersion") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaRelease) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaRelease
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaReleaseNotes: Release notes for a release.
type GoogleFirebaseAppdistroV1alphaReleaseNotes struct {
	// ReleaseNotes: The actual release notes text from the user.
	ReleaseNotes string `json:"releaseNotes,omitempty"`
	// ForceSendFields is a list of field names (e.g. "ReleaseNotes") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "ReleaseNotes") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaReleaseNotes) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaReleaseNotes
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaReleaseTest: Instance of an automated test for
// a release.
type GoogleFirebaseAppdistroV1alphaReleaseTest struct {
	// AiInstructions: Optional. Instructions for AI driven test.
	AiInstructions *GoogleFirebaseAppdistroV1alphaAiInstructions `json:"aiInstructions,omitempty"`
	// CreateTime: Output only. Timestamp when the test was run.
	CreateTime string `json:"createTime,omitempty"`
	// DeviceExecutions: Required. The results of the test on each device.
	DeviceExecutions []*GoogleFirebaseAppdistroV1alphaDeviceExecution `json:"deviceExecutions,omitempty"`
	// DisplayName: Optional. Display name of the release test. Required if the
	// release test is created with multiple goals.
	DisplayName string `json:"displayName,omitempty"`
	// LoginCredential: Optional. Input only. Login credentials for the test. Input
	// only.
	LoginCredential *GoogleFirebaseAppdistroV1alphaLoginCredential `json:"loginCredential,omitempty"`
	// Name: The name of the release test resource. Format:
	// `projects/{project_number}/apps/{app}/releases/{release}/tests/{test}`
	Name string `json:"name,omitempty"`
	// TestCase: Optional. The test case that was used to generate this release
	// test. Note: The test case may have changed or been deleted since the release
	// test was created. Format:
	// `projects/{project_number}/apps/{app}/testCases/{test_case}`
	TestCase string `json:"testCase,omitempty"`
	// TestState: Output only. The state of the release test.
	//
	// Possible values:
	//   "TEST_STATE_UNSPECIFIED" - Test state unspecified.
	//   "IN_PROGRESS" - The test is in progress.
	//   "PASSED" - The test has passed.
	//   "FAILED" - The test has failed.
	//   "INCONCLUSIVE" - The test was inconclusive.
	TestState string `json:"testState,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "AiInstructions") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "AiInstructions") to include in
	// API requests with the JSON null value. By default, fields with empty values
	// are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaReleaseTest) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaReleaseTest
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaRoboCrawler: Configuration for Robo crawler
type GoogleFirebaseAppdistroV1alphaRoboCrawler struct {
	// AiInstructions: Optional. Instructions for AI driven test
	AiInstructions *GoogleFirebaseAppdistroV1alphaAiInstructions `json:"aiInstructions,omitempty"`
	// LoginCredential: Optional. Login credential for automated tests
	LoginCredential *GoogleFirebaseAppdistroV1alphaLoginCredential `json:"loginCredential,omitempty"`
	// ForceSendFields is a list of field names (e.g. "AiInstructions") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "AiInstructions") to include in
	// API requests with the JSON null value. By default, fields with empty values
	// are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaRoboCrawler) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaRoboCrawler
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaRoboStats: Statistics collected during a Robo
// test.
type GoogleFirebaseAppdistroV1alphaRoboStats struct {
	// ActionsPerformed: Output only. Number of actions that crawler performed.
	ActionsPerformed int64 `json:"actionsPerformed,omitempty"`
	// CrawlDuration: Output only. Duration of crawl.
	CrawlDuration string `json:"crawlDuration,omitempty"`
	// DistinctVisitedScreens: Output only. Number of distinct screens visited.
	DistinctVisitedScreens int64 `json:"distinctVisitedScreens,omitempty"`
	// MainActivityCrawlTimedOut: Output only. Whether the main activity crawl
	// timed out.
	MainActivityCrawlTimedOut bool `json:"mainActivityCrawlTimedOut,omitempty"`
	// ForceSendFields is a list of field names (e.g. "ActionsPerformed") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "ActionsPerformed") to include in
	// API requests with the JSON null value. By default, fields with empty values
	// are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaRoboStats) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaRoboStats
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaScreenshot: A device screenshot taken during a
// test.
type GoogleFirebaseAppdistroV1alphaScreenshot struct {
	// Height: Output only. The height of the screenshot, in pixels.
	Height int64 `json:"height,omitempty"`
	// Uri: Output only. The URI of the screenshot.
	Uri string `json:"uri,omitempty"`
	// Width: Output only. The width of the screenshot, in pixels.
	Width int64 `json:"width,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Height") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Height") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaScreenshot) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaScreenshot
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaTerminalAction: An action taken by the AI to
// end the goal.
type GoogleFirebaseAppdistroV1alphaTerminalAction struct {
	// Reason: Output only. The reason why this goal was ended.
	//
	// Possible values:
	//   "REASON_UNSPECIFIED" - Reason unspecified.
	//   "GOAL_IMPOSSIBLE" - The goal failed to be completed. Note that
	// "impossible" is a legacy term and the goal is reported to the customer as
	// having "failed".
	//   "GOAL_COMPLETE" - The goal was completed successfully.
	Reason string `json:"reason,omitempty"`
	// Screenshot: Output only. The screenshot used in the context of this terminal
	// action.
	Screenshot *GoogleFirebaseAppdistroV1alphaScreenshot `json:"screenshot,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Reason") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Reason") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaTerminalAction) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaTerminalAction
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaTestCase: AI test cases
type GoogleFirebaseAppdistroV1alphaTestCase struct {
	// AiInstructions: Optional. Instructions for AI driven test.
	AiInstructions *GoogleFirebaseAppdistroV1alphaAiInstructions `json:"aiInstructions,omitempty"`
	// CreateTime: Output only. Timestamp when the test case was created
	CreateTime string `json:"createTime,omitempty"`
	// DependentTestCases: Output only. Other test cases that depend on this test
	// case as a prerequisite.
	DependentTestCases []string `json:"dependentTestCases,omitempty"`
	// DisplayName: Required. Display name of the test case.
	DisplayName string `json:"displayName,omitempty"`
	// Name: Identifier. The name of the test case resource. Format:
	// `projects/{project_number}/apps/{app}/testCases/{test_case}`
	Name string `json:"name,omitempty"`
	// PrerequisiteTestCase: Optional. Test case that must be run before this test
	// case.
	PrerequisiteTestCase string `json:"prerequisiteTestCase,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "AiInstructions") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "AiInstructions") to include in
	// API requests with the JSON null value. By default, fields with empty values
	// are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaTestCase) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaTestCase
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaTestConfig: Configuration for automated tests
type GoogleFirebaseAppdistroV1alphaTestConfig struct {
	// DisplayName: Optional. Display name of the AI driven test. Required if the
	// release test is created with multiple goals.
	DisplayName string `json:"displayName,omitempty"`
	// Name: Identifier. The name of the test configuration resource. Format:
	// `projects/{project_number}/apps/{app}/testConfig`
	Name string `json:"name,omitempty"`
	// RoboCrawler: Optional. Configuration for Robo crawler
	RoboCrawler *GoogleFirebaseAppdistroV1alphaRoboCrawler `json:"roboCrawler,omitempty"`
	// TestDevices: Optional. Tests will be run on this list of devices
	TestDevices []*GoogleFirebaseAppdistroV1alphaTestDevice `json:"testDevices,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "DisplayName") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "DisplayName") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaTestConfig) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaTestConfig
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaTestDevice: A device on which automated tests
// can be run.
type GoogleFirebaseAppdistroV1alphaTestDevice struct {
	// Locale: Optional. The locale of the device (e.g. "en_US" for US English)
	// during the test.
	Locale string `json:"locale,omitempty"`
	// Model: Required. The device model.
	Model string `json:"model,omitempty"`
	// Orientation: Optional. The orientation of the device during the test.
	Orientation string `json:"orientation,omitempty"`
	// Version: Required. The version of the device (API level on Android).
	Version string `json:"version,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Locale") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Locale") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaTestDevice) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaTestDevice
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaTestQuota: Customer quota information for
// `ReleaseTests`. Note: This quota only applies to tests with `AiInstructions`
// and is separate from the quota which might apply to the device time used by
// any tests.
type GoogleFirebaseAppdistroV1alphaTestQuota struct {
	// Limit: Output only. Maximum number of `ReleaseTests` allotted for the
	// current month.
	Limit int64 `json:"limit,omitempty,string"`
	// Name: Identifier. The name of the `TestQuota` resource. Format:
	// `projects/{project_number}/testQuota`
	Name string `json:"name,omitempty"`
	// Usage: Output only. Number of `ReleaseTests` run in the current month
	Usage int64 `json:"usage,omitempty,string"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "Limit") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Limit") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaTestQuota) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaTestQuota
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaTesterUdid: The UDIDs of a tester's iOS device
type GoogleFirebaseAppdistroV1alphaTesterUdid struct {
	// Name: The name of the tester's device
	Name string `json:"name,omitempty"`
	// Platform: The platform of the tester's device
	Platform string `json:"platform,omitempty"`
	// Udid: The UDID of the tester's device
	Udid string `json:"udid,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Name") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Name") to include in API requests
	// with the JSON null value. By default, fields with empty values are omitted
	// from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaTesterUdid) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaTesterUdid
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleFirebaseAppdistroV1alphaUpdateTestCaseRequest: The request message for
// `UpdateTestCase`.
type GoogleFirebaseAppdistroV1alphaUpdateTestCaseRequest struct {
	// AllowMissing: Optional. If set to true, and the test case is not found, a
	// new test case will be created.
	AllowMissing bool `json:"allowMissing,omitempty"`
	// TestCase: Required. The test case to update. The test case's `name` field is
	// used to identify the test case to update. Format:
	// `projects/{project_number}/apps/{app}/testCases/{test_case}`
	TestCase *GoogleFirebaseAppdistroV1alphaTestCase `json:"testCase,omitempty"`
	// ForceSendFields is a list of field names (e.g. "AllowMissing") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "AllowMissing") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s GoogleFirebaseAppdistroV1alphaUpdateTestCaseRequest) MarshalJSON() ([]byte, error) {
	type NoMethod GoogleFirebaseAppdistroV1alphaUpdateTestCaseRequest
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// GoogleProtobufEmpty: A generic empty message that you can re-use to avoid
// defining duplicated empty messages in your APIs. A typical example is to use
// it as the request or the response type of an API method. For instance:
// service Foo { rpc Bar(google.protobuf.Empty) returns
// (google.protobuf.Empty); }
type GoogleProtobufEmpty struct {
	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
}

type AppsGetCall struct {
	s              *Service
	mobilesdkAppId string
	urlParams_     gensupport.URLParams
	ifNoneMatch_   string
	ctx_           context.Context
	header_        http.Header
}

// Get: Get the app, if it exists
//
//   - mobilesdkAppId: Unique id for a Firebase app of the format:
//     {version}:{project_number}:{platform}:{hash(bundle_id)} Example:
//     1:581234567376:android:aa0a3c7b135e90289.
func (r *AppsService) Get(mobilesdkAppId string) *AppsGetCall {
	c := &AppsGetCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.mobilesdkAppId = mobilesdkAppId
	return c
}

// AppView sets the optional parameter "appView": App view. When unset or set
// to BASIC, returns an App with everything set except for aab_state. When set
// to FULL, returns an App with aab_state set.
//
// Possible values:
//
//	"APP_VIEW_UNSPECIFIED" - The default / unset value. The API will default
//
// to the BASIC view.
//
//	"BASIC" - Include everything except aab_state.
//	"FULL" - Include everything.
func (c *AppsGetCall) AppView(appView string) *AppsGetCall {
	c.urlParams_.Set("appView", appView)
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *AppsGetCall) Fields(s ...googleapi.Field) *AppsGetCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *AppsGetCall) IfNoneMatch(entityTag string) *AppsGetCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *AppsGetCall) Context(ctx context.Context) *AppsGetCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *AppsGetCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *AppsGetCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/apps/{mobilesdkAppId}")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"mobilesdkAppId": c.mobilesdkAppId,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.get", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.apps.get" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaApp.ServerResponse.Header or (if a response
// was returned at all) in error.(*googleapi.Error).Header. Use
// googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *AppsGetCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaApp, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaApp{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.get", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type AppsGetJwtCall struct {
	s              *Service
	mobilesdkAppId string
	urlParams_     gensupport.URLParams
	ifNoneMatch_   string
	ctx_           context.Context
	header_        http.Header
}

// GetJwt: Get a JWT token
//
//   - mobilesdkAppId: Unique id for a Firebase app of the format:
//     {version}:{project_number}:{platform}:{hash(bundle_id)} Example:
//     1:581234567376:android:aa0a3c7b135e90289.
func (r *AppsService) GetJwt(mobilesdkAppId string) *AppsGetJwtCall {
	c := &AppsGetJwtCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.mobilesdkAppId = mobilesdkAppId
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *AppsGetJwtCall) Fields(s ...googleapi.Field) *AppsGetJwtCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *AppsGetJwtCall) IfNoneMatch(entityTag string) *AppsGetJwtCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *AppsGetJwtCall) Context(ctx context.Context) *AppsGetJwtCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *AppsGetJwtCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *AppsGetJwtCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/apps/{mobilesdkAppId}/jwt")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"mobilesdkAppId": c.mobilesdkAppId,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.getJwt", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.apps.getJwt" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaJwt.ServerResponse.Header or (if a response
// was returned at all) in error.(*googleapi.Error).Header. Use
// googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *AppsGetJwtCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaJwt, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaJwt{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.getJwt", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type AppsReleaseByHashGetCall struct {
	s              *Service
	mobilesdkAppId string
	uploadHash     string
	urlParams_     gensupport.URLParams
	ifNoneMatch_   string
	ctx_           context.Context
	header_        http.Header
}

// Get: GET Release by binary upload hash
//
//   - mobilesdkAppId: Unique id for a Firebase app of the format:
//     {version}:{project_number}:{platform}:{hash(bundle_id)} Example:
//     1:581234567376:android:aa0a3c7b135e90289.
//   - uploadHash: The hash for the upload.
func (r *AppsReleaseByHashService) Get(mobilesdkAppId string, uploadHash string) *AppsReleaseByHashGetCall {
	c := &AppsReleaseByHashGetCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.mobilesdkAppId = mobilesdkAppId
	c.uploadHash = uploadHash
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *AppsReleaseByHashGetCall) Fields(s ...googleapi.Field) *AppsReleaseByHashGetCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *AppsReleaseByHashGetCall) IfNoneMatch(entityTag string) *AppsReleaseByHashGetCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *AppsReleaseByHashGetCall) Context(ctx context.Context) *AppsReleaseByHashGetCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *AppsReleaseByHashGetCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *AppsReleaseByHashGetCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/apps/{mobilesdkAppId}/release_by_hash/{uploadHash}")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"mobilesdkAppId": c.mobilesdkAppId,
		"uploadHash":     c.uploadHash,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.release_by_hash.get", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.apps.release_by_hash.get" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaGetReleaseByUploadHashResponse.ServerResponse.
// Header or (if a response was returned at all) in
// error.(*googleapi.Error).Header. Use googleapi.IsNotModified to check
// whether the returned error was because http.StatusNotModified was returned.
func (c *AppsReleaseByHashGetCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaGetReleaseByUploadHashResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaGetReleaseByUploadHashResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.release_by_hash.get", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type AppsReleasesEnableAccessCall struct {
	s                                                          *Service
	mobilesdkAppId                                             string
	releaseId                                                  string
	googlefirebaseappdistrov1alphaenableaccessonreleaserequest *GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseRequest
	urlParams_                                                 gensupport.URLParams
	ctx_                                                       context.Context
	header_                                                    http.Header
}

// EnableAccess: Enable access on a release for testers.
//
//   - mobilesdkAppId: Unique id for a Firebase app of the format:
//     {version}:{project_number}:{platform}:{hash(bundle_id)} Example:
//     1:581234567376:android:aa0a3c7b135e90289.
//   - releaseId: Release identifier.
func (r *AppsReleasesService) EnableAccess(mobilesdkAppId string, releaseId string, googlefirebaseappdistrov1alphaenableaccessonreleaserequest *GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseRequest) *AppsReleasesEnableAccessCall {
	c := &AppsReleasesEnableAccessCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.mobilesdkAppId = mobilesdkAppId
	c.releaseId = releaseId
	c.googlefirebaseappdistrov1alphaenableaccessonreleaserequest = googlefirebaseappdistrov1alphaenableaccessonreleaserequest
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *AppsReleasesEnableAccessCall) Fields(s ...googleapi.Field) *AppsReleasesEnableAccessCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *AppsReleasesEnableAccessCall) Context(ctx context.Context) *AppsReleasesEnableAccessCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *AppsReleasesEnableAccessCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *AppsReleasesEnableAccessCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.googlefirebaseappdistrov1alphaenableaccessonreleaserequest)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/apps/{mobilesdkAppId}/releases/{releaseId}/enable_access")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("POST", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"mobilesdkAppId": c.mobilesdkAppId,
		"releaseId":      c.releaseId,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.releases.enable_access", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.apps.releases.enable_access" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseResponse.ServerResponse.H
// eader or (if a response was returned at all) in
// error.(*googleapi.Error).Header. Use googleapi.IsNotModified to check
// whether the returned error was because http.StatusNotModified was returned.
func (c *AppsReleasesEnableAccessCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaEnableAccessOnReleaseResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.releases.enable_access", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type AppsReleasesNotesCreateCall struct {
	s                                                       *Service
	mobilesdkAppId                                          string
	releaseId                                               string
	googlefirebaseappdistrov1alphacreatereleasenotesrequest *GoogleFirebaseAppdistroV1alphaCreateReleaseNotesRequest
	urlParams_                                              gensupport.URLParams
	ctx_                                                    context.Context
	header_                                                 http.Header
}

// Create: Create release notes on a release.
//
//   - mobilesdkAppId: Unique id for a Firebase app of the format:
//     {version}:{project_number}:{platform}:{hash(bundle_id)} Example:
//     1:581234567376:android:aa0a3c7b135e90289.
//   - releaseId: Release identifier.
func (r *AppsReleasesNotesService) Create(mobilesdkAppId string, releaseId string, googlefirebaseappdistrov1alphacreatereleasenotesrequest *GoogleFirebaseAppdistroV1alphaCreateReleaseNotesRequest) *AppsReleasesNotesCreateCall {
	c := &AppsReleasesNotesCreateCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.mobilesdkAppId = mobilesdkAppId
	c.releaseId = releaseId
	c.googlefirebaseappdistrov1alphacreatereleasenotesrequest = googlefirebaseappdistrov1alphacreatereleasenotesrequest
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *AppsReleasesNotesCreateCall) Fields(s ...googleapi.Field) *AppsReleasesNotesCreateCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *AppsReleasesNotesCreateCall) Context(ctx context.Context) *AppsReleasesNotesCreateCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *AppsReleasesNotesCreateCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *AppsReleasesNotesCreateCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.googlefirebaseappdistrov1alphacreatereleasenotesrequest)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/apps/{mobilesdkAppId}/releases/{releaseId}/notes")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("POST", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"mobilesdkAppId": c.mobilesdkAppId,
		"releaseId":      c.releaseId,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.releases.notes.create", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.apps.releases.notes.create" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaCreateReleaseNotesResponse.ServerResponse.Head
// er or (if a response was returned at all) in
// error.(*googleapi.Error).Header. Use googleapi.IsNotModified to check
// whether the returned error was because http.StatusNotModified was returned.
func (c *AppsReleasesNotesCreateCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaCreateReleaseNotesResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaCreateReleaseNotesResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.releases.notes.create", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type AppsTestersGetTesterUdidsCall struct {
	s              *Service
	mobilesdkAppId string
	urlParams_     gensupport.URLParams
	ifNoneMatch_   string
	ctx_           context.Context
	header_        http.Header
}

// GetTesterUdids: Get UDIDs of tester iOS devices in a project
//
//   - mobilesdkAppId: Unique id for a Firebase app of the format:
//     {version}:{project_number}:{platform}:{hash(bundle_id)} Example:
//     1:581234567376:android:aa0a3c7b135e90289.
func (r *AppsTestersService) GetTesterUdids(mobilesdkAppId string) *AppsTestersGetTesterUdidsCall {
	c := &AppsTestersGetTesterUdidsCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.mobilesdkAppId = mobilesdkAppId
	return c
}

// Project sets the optional parameter "project": The name of the project,
// which is the parent of testers Format: `projects/{project_number}`
func (c *AppsTestersGetTesterUdidsCall) Project(project string) *AppsTestersGetTesterUdidsCall {
	c.urlParams_.Set("project", project)
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *AppsTestersGetTesterUdidsCall) Fields(s ...googleapi.Field) *AppsTestersGetTesterUdidsCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *AppsTestersGetTesterUdidsCall) IfNoneMatch(entityTag string) *AppsTestersGetTesterUdidsCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *AppsTestersGetTesterUdidsCall) Context(ctx context.Context) *AppsTestersGetTesterUdidsCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *AppsTestersGetTesterUdidsCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *AppsTestersGetTesterUdidsCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/apps/{mobilesdkAppId}/testers:getTesterUdids")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"mobilesdkAppId": c.mobilesdkAppId,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.testers.getTesterUdids", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.apps.testers.getTesterUdids" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaGetTesterUdidsResponse.ServerResponse.Header
// or (if a response was returned at all) in error.(*googleapi.Error).Header.
// Use googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *AppsTestersGetTesterUdidsCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaGetTesterUdidsResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaGetTesterUdidsResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.testers.getTesterUdids", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type AppsUploadStatusGetCall struct {
	s              *Service
	mobilesdkAppId string
	uploadToken    string
	urlParams_     gensupport.URLParams
	ifNoneMatch_   string
	ctx_           context.Context
	header_        http.Header
}

// Get: GET Binary upload status by token
//
//   - mobilesdkAppId: Unique id for a Firebase app of the format:
//     {version}:{project_number}:{platform}:{hash(bundle_id)} Example:
//     1:581234567376:android:aa0a3c7b135e90289.
//   - uploadToken: The token for the upload.
func (r *AppsUploadStatusService) Get(mobilesdkAppId string, uploadToken string) *AppsUploadStatusGetCall {
	c := &AppsUploadStatusGetCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.mobilesdkAppId = mobilesdkAppId
	c.uploadToken = uploadToken
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *AppsUploadStatusGetCall) Fields(s ...googleapi.Field) *AppsUploadStatusGetCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *AppsUploadStatusGetCall) IfNoneMatch(entityTag string) *AppsUploadStatusGetCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *AppsUploadStatusGetCall) Context(ctx context.Context) *AppsUploadStatusGetCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *AppsUploadStatusGetCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *AppsUploadStatusGetCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/apps/{mobilesdkAppId}/upload_status/{uploadToken}")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"mobilesdkAppId": c.mobilesdkAppId,
		"uploadToken":    c.uploadToken,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.upload_status.get", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.apps.upload_status.get" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaGetUploadStatusResponse.ServerResponse.Header
// or (if a response was returned at all) in error.(*googleapi.Error).Header.
// Use googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *AppsUploadStatusGetCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaGetUploadStatusResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaGetUploadStatusResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.apps.upload_status.get", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsGetTestQuotaCall struct {
	s            *Service
	name         string
	urlParams_   gensupport.URLParams
	ifNoneMatch_ string
	ctx_         context.Context
	header_      http.Header
}

// GetTestQuota: Get information about the quota for `ReleaseTests`.
//
//   - name: The name of the `TestQuota` resource to retrieve. Format:
//     `projects/{project_number}/testQuota`.
func (r *ProjectsService) GetTestQuota(name string) *ProjectsGetTestQuotaCall {
	c := &ProjectsGetTestQuotaCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.name = name
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsGetTestQuotaCall) Fields(s ...googleapi.Field) *ProjectsGetTestQuotaCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *ProjectsGetTestQuotaCall) IfNoneMatch(entityTag string) *ProjectsGetTestQuotaCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsGetTestQuotaCall) Context(ctx context.Context) *ProjectsGetTestQuotaCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsGetTestQuotaCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsGetTestQuotaCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+name}")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"name": c.name,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.getTestQuota", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.getTestQuota" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaTestQuota.ServerResponse.Header or (if a
// response was returned at all) in error.(*googleapi.Error).Header. Use
// googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsGetTestQuotaCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaTestQuota, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaTestQuota{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.getTestQuota", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsGetTestConfigCall struct {
	s            *Service
	name         string
	urlParams_   gensupport.URLParams
	ifNoneMatch_ string
	ctx_         context.Context
	header_      http.Header
}

// GetTestConfig: Gets configuration for automated tests.
//
//   - name: The name of the `TestConfig` resource to retrieve. Format:
//     `projects/{project_number}/apps/{app}/testConfig`.
func (r *ProjectsAppsService) GetTestConfig(name string) *ProjectsAppsGetTestConfigCall {
	c := &ProjectsAppsGetTestConfigCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.name = name
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsGetTestConfigCall) Fields(s ...googleapi.Field) *ProjectsAppsGetTestConfigCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *ProjectsAppsGetTestConfigCall) IfNoneMatch(entityTag string) *ProjectsAppsGetTestConfigCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsGetTestConfigCall) Context(ctx context.Context) *ProjectsAppsGetTestConfigCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsGetTestConfigCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsGetTestConfigCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+name}")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"name": c.name,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.getTestConfig", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.getTestConfig" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaTestConfig.ServerResponse.Header or (if a
// response was returned at all) in error.(*googleapi.Error).Header. Use
// googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsAppsGetTestConfigCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaTestConfig, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaTestConfig{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.getTestConfig", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsUpdateTestConfigCall struct {
	s                                        *Service
	name                                     string
	googlefirebaseappdistrov1alphatestconfig *GoogleFirebaseAppdistroV1alphaTestConfig
	urlParams_                               gensupport.URLParams
	ctx_                                     context.Context
	header_                                  http.Header
}

// UpdateTestConfig: Updates automated test configuration.
//
//   - name: Identifier. The name of the test configuration resource. Format:
//     `projects/{project_number}/apps/{app}/testConfig`.
func (r *ProjectsAppsService) UpdateTestConfig(name string, googlefirebaseappdistrov1alphatestconfig *GoogleFirebaseAppdistroV1alphaTestConfig) *ProjectsAppsUpdateTestConfigCall {
	c := &ProjectsAppsUpdateTestConfigCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.name = name
	c.googlefirebaseappdistrov1alphatestconfig = googlefirebaseappdistrov1alphatestconfig
	return c
}

// UpdateMask sets the optional parameter "updateMask": The list of fields to
// update.
func (c *ProjectsAppsUpdateTestConfigCall) UpdateMask(updateMask string) *ProjectsAppsUpdateTestConfigCall {
	c.urlParams_.Set("updateMask", updateMask)
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsUpdateTestConfigCall) Fields(s ...googleapi.Field) *ProjectsAppsUpdateTestConfigCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsUpdateTestConfigCall) Context(ctx context.Context) *ProjectsAppsUpdateTestConfigCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsUpdateTestConfigCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsUpdateTestConfigCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.googlefirebaseappdistrov1alphatestconfig)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+name}")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("PATCH", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"name": c.name,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.updateTestConfig", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.updateTestConfig" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaTestConfig.ServerResponse.Header or (if a
// response was returned at all) in error.(*googleapi.Error).Header. Use
// googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsAppsUpdateTestConfigCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaTestConfig, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaTestConfig{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.updateTestConfig", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsReleasesTestsCancelCall struct {
	s            *Service
	name         string
	urlParams_   gensupport.URLParams
	ifNoneMatch_ string
	ctx_         context.Context
	header_      http.Header
}

// Cancel: Abort automated test run on release.
//
//   - name: The name of the release test resource. Format:
//     `projects/{project_number}/apps/{app}/releases/{release}/tests/{test}`.
func (r *ProjectsAppsReleasesTestsService) Cancel(name string) *ProjectsAppsReleasesTestsCancelCall {
	c := &ProjectsAppsReleasesTestsCancelCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.name = name
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsReleasesTestsCancelCall) Fields(s ...googleapi.Field) *ProjectsAppsReleasesTestsCancelCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *ProjectsAppsReleasesTestsCancelCall) IfNoneMatch(entityTag string) *ProjectsAppsReleasesTestsCancelCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsReleasesTestsCancelCall) Context(ctx context.Context) *ProjectsAppsReleasesTestsCancelCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsReleasesTestsCancelCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsReleasesTestsCancelCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+name}:cancel")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"name": c.name,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.releases.tests.cancel", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.releases.tests.cancel" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaCancelReleaseTestResponse.ServerResponse.Heade
// r or (if a response was returned at all) in error.(*googleapi.Error).Header.
// Use googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsAppsReleasesTestsCancelCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaCancelReleaseTestResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaCancelReleaseTestResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.releases.tests.cancel", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsReleasesTestsCreateCall struct {
	s                                         *Service
	parent                                    string
	googlefirebaseappdistrov1alphareleasetest *GoogleFirebaseAppdistroV1alphaReleaseTest
	urlParams_                                gensupport.URLParams
	ctx_                                      context.Context
	header_                                   http.Header
}

// Create: Run automated test(s) on release.
//
//   - parent: The name of the release resource, which is the parent of the test
//     Format: `projects/{project_number}/apps/{app}/releases/{release}`.
func (r *ProjectsAppsReleasesTestsService) Create(parent string, googlefirebaseappdistrov1alphareleasetest *GoogleFirebaseAppdistroV1alphaReleaseTest) *ProjectsAppsReleasesTestsCreateCall {
	c := &ProjectsAppsReleasesTestsCreateCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.parent = parent
	c.googlefirebaseappdistrov1alphareleasetest = googlefirebaseappdistrov1alphareleasetest
	return c
}

// ReleaseTestId sets the optional parameter "releaseTestId": The ID to use for
// the test, which will become the final component of the test's resource name.
// This value should be 4-63 characters, and valid characters are /a-z-/. If it
// is not provided one will be automatically generated.
func (c *ProjectsAppsReleasesTestsCreateCall) ReleaseTestId(releaseTestId string) *ProjectsAppsReleasesTestsCreateCall {
	c.urlParams_.Set("releaseTestId", releaseTestId)
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsReleasesTestsCreateCall) Fields(s ...googleapi.Field) *ProjectsAppsReleasesTestsCreateCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsReleasesTestsCreateCall) Context(ctx context.Context) *ProjectsAppsReleasesTestsCreateCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsReleasesTestsCreateCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsReleasesTestsCreateCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.googlefirebaseappdistrov1alphareleasetest)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+parent}/tests")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("POST", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"parent": c.parent,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.releases.tests.create", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.releases.tests.create" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaReleaseTest.ServerResponse.Header or (if a
// response was returned at all) in error.(*googleapi.Error).Header. Use
// googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsAppsReleasesTestsCreateCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaReleaseTest, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaReleaseTest{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.releases.tests.create", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsReleasesTestsGetCall struct {
	s            *Service
	name         string
	urlParams_   gensupport.URLParams
	ifNoneMatch_ string
	ctx_         context.Context
	header_      http.Header
}

// Get: Get results for automated test run on release.
//
//   - name: The name of the release test resource. Format:
//     `projects/{project_number}/apps/{app}/releases/{release}/tests/{test}`.
func (r *ProjectsAppsReleasesTestsService) Get(name string) *ProjectsAppsReleasesTestsGetCall {
	c := &ProjectsAppsReleasesTestsGetCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.name = name
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsReleasesTestsGetCall) Fields(s ...googleapi.Field) *ProjectsAppsReleasesTestsGetCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *ProjectsAppsReleasesTestsGetCall) IfNoneMatch(entityTag string) *ProjectsAppsReleasesTestsGetCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsReleasesTestsGetCall) Context(ctx context.Context) *ProjectsAppsReleasesTestsGetCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsReleasesTestsGetCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsReleasesTestsGetCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+name}")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"name": c.name,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.releases.tests.get", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.releases.tests.get" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaReleaseTest.ServerResponse.Header or (if a
// response was returned at all) in error.(*googleapi.Error).Header. Use
// googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsAppsReleasesTestsGetCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaReleaseTest, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaReleaseTest{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.releases.tests.get", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsReleasesTestsListCall struct {
	s            *Service
	parent       string
	urlParams_   gensupport.URLParams
	ifNoneMatch_ string
	ctx_         context.Context
	header_      http.Header
}

// List: List results for automated tests run on release.
//
//   - parent: The name of the release resource, which is the parent of the tests
//     Format: `projects/{project_number}/apps/{app}/releases/{release}`.
func (r *ProjectsAppsReleasesTestsService) List(parent string) *ProjectsAppsReleasesTestsListCall {
	c := &ProjectsAppsReleasesTestsListCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.parent = parent
	return c
}

// PageSize sets the optional parameter "pageSize": The maximum number of tests
// to return. The service may return fewer than this value.
func (c *ProjectsAppsReleasesTestsListCall) PageSize(pageSize int64) *ProjectsAppsReleasesTestsListCall {
	c.urlParams_.Set("pageSize", fmt.Sprint(pageSize))
	return c
}

// PageToken sets the optional parameter "pageToken": A page token, received
// from a previous `ListReleaseTests` call. Provide this to retrieve the
// subsequent page.
func (c *ProjectsAppsReleasesTestsListCall) PageToken(pageToken string) *ProjectsAppsReleasesTestsListCall {
	c.urlParams_.Set("pageToken", pageToken)
	return c
}

// View sets the optional parameter "view": The requested view on the returned
// ReleaseTests. Defaults to the basic view.
//
// Possible values:
//
//	"RELEASE_TEST_VIEW_UNSPECIFIED" - The default / unset value. The default
//
// view depends on the RPC.
//
//	"RELEASE_TEST_VIEW_BASIC" - Include basic metadata about the release test
//
// and its status, but not the full result details. This is the default value
// for ListReleaseTests.
//
//	"RELEASE_TEST_VIEW_FULL" - Include everything.
func (c *ProjectsAppsReleasesTestsListCall) View(view string) *ProjectsAppsReleasesTestsListCall {
	c.urlParams_.Set("view", view)
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsReleasesTestsListCall) Fields(s ...googleapi.Field) *ProjectsAppsReleasesTestsListCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *ProjectsAppsReleasesTestsListCall) IfNoneMatch(entityTag string) *ProjectsAppsReleasesTestsListCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsReleasesTestsListCall) Context(ctx context.Context) *ProjectsAppsReleasesTestsListCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsReleasesTestsListCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsReleasesTestsListCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+parent}/tests")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"parent": c.parent,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.releases.tests.list", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.releases.tests.list" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaListReleaseTestsResponse.ServerResponse.Header
//
//	or (if a response was returned at all) in error.(*googleapi.Error).Header.
//
// Use googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsAppsReleasesTestsListCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaListReleaseTestsResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaListReleaseTestsResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.releases.tests.list", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

// Pages invokes f for each page of results.
// A non-nil error returned from f will halt the iteration.
// The provided context supersedes any context provided to the Context method.
func (c *ProjectsAppsReleasesTestsListCall) Pages(ctx context.Context, f func(*GoogleFirebaseAppdistroV1alphaListReleaseTestsResponse) error) error {
	c.ctx_ = ctx
	defer c.PageToken(c.urlParams_.Get("pageToken"))
	for {
		x, err := c.Do()
		if err != nil {
			return err
		}
		if err := f(x); err != nil {
			return err
		}
		if x.NextPageToken == "" {
			return nil
		}
		c.PageToken(x.NextPageToken)
	}
}

type ProjectsAppsTestCasesBatchDeleteCall struct {
	s                                                         *Service
	parent                                                    string
	googlefirebaseappdistrov1alphabatchdeletetestcasesrequest *GoogleFirebaseAppdistroV1alphaBatchDeleteTestCasesRequest
	urlParams_                                                gensupport.URLParams
	ctx_                                                      context.Context
	header_                                                   http.Header
}

// BatchDelete: Deletes multiple test cases.
//
//   - parent: The parent resource of the test cases being deleted. Format:
//     `projects/{project_number}/apps/{app}`.
func (r *ProjectsAppsTestCasesService) BatchDelete(parent string, googlefirebaseappdistrov1alphabatchdeletetestcasesrequest *GoogleFirebaseAppdistroV1alphaBatchDeleteTestCasesRequest) *ProjectsAppsTestCasesBatchDeleteCall {
	c := &ProjectsAppsTestCasesBatchDeleteCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.parent = parent
	c.googlefirebaseappdistrov1alphabatchdeletetestcasesrequest = googlefirebaseappdistrov1alphabatchdeletetestcasesrequest
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsTestCasesBatchDeleteCall) Fields(s ...googleapi.Field) *ProjectsAppsTestCasesBatchDeleteCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsTestCasesBatchDeleteCall) Context(ctx context.Context) *ProjectsAppsTestCasesBatchDeleteCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsTestCasesBatchDeleteCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsTestCasesBatchDeleteCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.googlefirebaseappdistrov1alphabatchdeletetestcasesrequest)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+parent}/testCases:batchDelete")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("POST", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"parent": c.parent,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.batchDelete", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.testCases.batchDelete" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleProtobufEmpty.ServerResponse.Header or (if a response was returned at
// all) in error.(*googleapi.Error).Header. Use googleapi.IsNotModified to
// check whether the returned error was because http.StatusNotModified was
// returned.
func (c *ProjectsAppsTestCasesBatchDeleteCall) Do(opts ...googleapi.CallOption) (*GoogleProtobufEmpty, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleProtobufEmpty{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.batchDelete", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsTestCasesBatchUpdateCall struct {
	s                                                         *Service
	parent                                                    string
	googlefirebaseappdistrov1alphabatchupdatetestcasesrequest *GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesRequest
	urlParams_                                                gensupport.URLParams
	ctx_                                                      context.Context
	header_                                                   http.Header
}

// BatchUpdate: Updates multiple test cases.
//
//   - parent: The parent resource of the test cases being updated. Format:
//     `projects/{project_number}/apps/{app}`.
func (r *ProjectsAppsTestCasesService) BatchUpdate(parent string, googlefirebaseappdistrov1alphabatchupdatetestcasesrequest *GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesRequest) *ProjectsAppsTestCasesBatchUpdateCall {
	c := &ProjectsAppsTestCasesBatchUpdateCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.parent = parent
	c.googlefirebaseappdistrov1alphabatchupdatetestcasesrequest = googlefirebaseappdistrov1alphabatchupdatetestcasesrequest
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsTestCasesBatchUpdateCall) Fields(s ...googleapi.Field) *ProjectsAppsTestCasesBatchUpdateCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsTestCasesBatchUpdateCall) Context(ctx context.Context) *ProjectsAppsTestCasesBatchUpdateCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsTestCasesBatchUpdateCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsTestCasesBatchUpdateCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.googlefirebaseappdistrov1alphabatchupdatetestcasesrequest)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+parent}/testCases:batchUpdate")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("POST", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"parent": c.parent,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.batchUpdate", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.testCases.batchUpdate" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesResponse.ServerResponse.He
// ader or (if a response was returned at all) in
// error.(*googleapi.Error).Header. Use googleapi.IsNotModified to check
// whether the returned error was because http.StatusNotModified was returned.
func (c *ProjectsAppsTestCasesBatchUpdateCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaBatchUpdateTestCasesResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.batchUpdate", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsTestCasesClearTestCaseCacheCall struct {
	s                                                       *Service
	testCase                                                string
	googlefirebaseappdistrov1alphacleartestcasecacherequest *GoogleFirebaseAppdistroV1alphaClearTestCaseCacheRequest
	urlParams_                                              gensupport.URLParams
	ctx_                                                    context.Context
	header_                                                 http.Header
}

// ClearTestCaseCache: Clears cached test runs for a specific test case and
// device(s).
//
//   - testCase: The name of the test case resource for which to clear the cache.
//     Format: `projects/{project_number}/apps/{app}/testCases/{test_case}`.
func (r *ProjectsAppsTestCasesService) ClearTestCaseCache(testCase string, googlefirebaseappdistrov1alphacleartestcasecacherequest *GoogleFirebaseAppdistroV1alphaClearTestCaseCacheRequest) *ProjectsAppsTestCasesClearTestCaseCacheCall {
	c := &ProjectsAppsTestCasesClearTestCaseCacheCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.testCase = testCase
	c.googlefirebaseappdistrov1alphacleartestcasecacherequest = googlefirebaseappdistrov1alphacleartestcasecacherequest
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsTestCasesClearTestCaseCacheCall) Fields(s ...googleapi.Field) *ProjectsAppsTestCasesClearTestCaseCacheCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsTestCasesClearTestCaseCacheCall) Context(ctx context.Context) *ProjectsAppsTestCasesClearTestCaseCacheCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsTestCasesClearTestCaseCacheCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsTestCasesClearTestCaseCacheCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.googlefirebaseappdistrov1alphacleartestcasecacherequest)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+testCase}:clearTestCaseCache")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("POST", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"testCase": c.testCase,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.clearTestCaseCache", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.testCases.clearTestCaseCache" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaClearTestCaseCacheResponse.ServerResponse.Head
// er or (if a response was returned at all) in
// error.(*googleapi.Error).Header. Use googleapi.IsNotModified to check
// whether the returned error was because http.StatusNotModified was returned.
func (c *ProjectsAppsTestCasesClearTestCaseCacheCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaClearTestCaseCacheResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaClearTestCaseCacheResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.clearTestCaseCache", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsTestCasesCreateCall struct {
	s                                      *Service
	parent                                 string
	googlefirebaseappdistrov1alphatestcase *GoogleFirebaseAppdistroV1alphaTestCase
	urlParams_                             gensupport.URLParams
	ctx_                                   context.Context
	header_                                http.Header
}

// Create: Create a new test case.
//
//   - parent: The parent resource where this test case will be created. Format:
//     `projects/{project_number}/apps/{app}`.
func (r *ProjectsAppsTestCasesService) Create(parent string, googlefirebaseappdistrov1alphatestcase *GoogleFirebaseAppdistroV1alphaTestCase) *ProjectsAppsTestCasesCreateCall {
	c := &ProjectsAppsTestCasesCreateCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.parent = parent
	c.googlefirebaseappdistrov1alphatestcase = googlefirebaseappdistrov1alphatestcase
	return c
}

// TestCaseId sets the optional parameter "testCaseId": The ID to use for the
// test case, which will become the final component of the test case's resource
// name. This value should be 4-63 characters, and valid characters are /a-z-/.
func (c *ProjectsAppsTestCasesCreateCall) TestCaseId(testCaseId string) *ProjectsAppsTestCasesCreateCall {
	c.urlParams_.Set("testCaseId", testCaseId)
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsTestCasesCreateCall) Fields(s ...googleapi.Field) *ProjectsAppsTestCasesCreateCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsTestCasesCreateCall) Context(ctx context.Context) *ProjectsAppsTestCasesCreateCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsTestCasesCreateCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsTestCasesCreateCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.googlefirebaseappdistrov1alphatestcase)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+parent}/testCases")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("POST", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"parent": c.parent,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.create", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.testCases.create" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaTestCase.ServerResponse.Header or (if a
// response was returned at all) in error.(*googleapi.Error).Header. Use
// googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsAppsTestCasesCreateCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaTestCase, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaTestCase{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.create", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsTestCasesDeleteCall struct {
	s          *Service
	name       string
	urlParams_ gensupport.URLParams
	ctx_       context.Context
	header_    http.Header
}

// Delete: Delete a test case.
//
//   - name: The name of the test case resource to delete. Format:
//     `projects/{project_number}/apps/{app}/testCases/{test_case}`.
func (r *ProjectsAppsTestCasesService) Delete(name string) *ProjectsAppsTestCasesDeleteCall {
	c := &ProjectsAppsTestCasesDeleteCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.name = name
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsTestCasesDeleteCall) Fields(s ...googleapi.Field) *ProjectsAppsTestCasesDeleteCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsTestCasesDeleteCall) Context(ctx context.Context) *ProjectsAppsTestCasesDeleteCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsTestCasesDeleteCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsTestCasesDeleteCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+name}")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("DELETE", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"name": c.name,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.delete", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.testCases.delete" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleProtobufEmpty.ServerResponse.Header or (if a response was returned at
// all) in error.(*googleapi.Error).Header. Use googleapi.IsNotModified to
// check whether the returned error was because http.StatusNotModified was
// returned.
func (c *ProjectsAppsTestCasesDeleteCall) Do(opts ...googleapi.CallOption) (*GoogleProtobufEmpty, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleProtobufEmpty{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.delete", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsTestCasesGetCall struct {
	s            *Service
	name         string
	urlParams_   gensupport.URLParams
	ifNoneMatch_ string
	ctx_         context.Context
	header_      http.Header
}

// Get: Get a test case.
//
//   - name: The name of the test case resource to retrieve. Format:
//     `projects/{project_number}/apps/{app}/testCases/{test_case}`.
func (r *ProjectsAppsTestCasesService) Get(name string) *ProjectsAppsTestCasesGetCall {
	c := &ProjectsAppsTestCasesGetCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.name = name
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsTestCasesGetCall) Fields(s ...googleapi.Field) *ProjectsAppsTestCasesGetCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *ProjectsAppsTestCasesGetCall) IfNoneMatch(entityTag string) *ProjectsAppsTestCasesGetCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsTestCasesGetCall) Context(ctx context.Context) *ProjectsAppsTestCasesGetCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsTestCasesGetCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsTestCasesGetCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+name}")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"name": c.name,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.get", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.testCases.get" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaTestCase.ServerResponse.Header or (if a
// response was returned at all) in error.(*googleapi.Error).Header. Use
// googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsAppsTestCasesGetCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaTestCase, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaTestCase{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.get", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsAppsTestCasesListCall struct {
	s            *Service
	parent       string
	urlParams_   gensupport.URLParams
	ifNoneMatch_ string
	ctx_         context.Context
	header_      http.Header
}

// List: List test cases.
//
//   - parent: The parent resource from which to list test cases. Format:
//     `projects/{project_number}/apps/{app}`.
func (r *ProjectsAppsTestCasesService) List(parent string) *ProjectsAppsTestCasesListCall {
	c := &ProjectsAppsTestCasesListCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.parent = parent
	return c
}

// PageSize sets the optional parameter "pageSize": The maximum number of test
// cases to return. The service may return fewer than this value. If
// unspecified, at most 50 test cases will be returned. The maximum value is
// 1000; values above 1000 will be coerced to 1000.
func (c *ProjectsAppsTestCasesListCall) PageSize(pageSize int64) *ProjectsAppsTestCasesListCall {
	c.urlParams_.Set("pageSize", fmt.Sprint(pageSize))
	return c
}

// PageToken sets the optional parameter "pageToken": A page token, received
// from a previous `ListTestCases` call. Provide this to retrieve the
// subsequent page. When paginating, all other parameters provided to
// `ListTestCases` must match the call that provided the page token.
func (c *ProjectsAppsTestCasesListCall) PageToken(pageToken string) *ProjectsAppsTestCasesListCall {
	c.urlParams_.Set("pageToken", pageToken)
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsTestCasesListCall) Fields(s ...googleapi.Field) *ProjectsAppsTestCasesListCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *ProjectsAppsTestCasesListCall) IfNoneMatch(entityTag string) *ProjectsAppsTestCasesListCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsTestCasesListCall) Context(ctx context.Context) *ProjectsAppsTestCasesListCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsTestCasesListCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsTestCasesListCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+parent}/testCases")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"parent": c.parent,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.list", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.testCases.list" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaListTestCasesResponse.ServerResponse.Header
// or (if a response was returned at all) in error.(*googleapi.Error).Header.
// Use googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsAppsTestCasesListCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaListTestCasesResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaListTestCasesResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.list", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

// Pages invokes f for each page of results.
// A non-nil error returned from f will halt the iteration.
// The provided context supersedes any context provided to the Context method.
func (c *ProjectsAppsTestCasesListCall) Pages(ctx context.Context, f func(*GoogleFirebaseAppdistroV1alphaListTestCasesResponse) error) error {
	c.ctx_ = ctx
	defer c.PageToken(c.urlParams_.Get("pageToken"))
	for {
		x, err := c.Do()
		if err != nil {
			return err
		}
		if err := f(x); err != nil {
			return err
		}
		if x.NextPageToken == "" {
			return nil
		}
		c.PageToken(x.NextPageToken)
	}
}

type ProjectsAppsTestCasesPatchCall struct {
	s                                      *Service
	name                                   string
	googlefirebaseappdistrov1alphatestcase *GoogleFirebaseAppdistroV1alphaTestCase
	urlParams_                             gensupport.URLParams
	ctx_                                   context.Context
	header_                                http.Header
}

// Patch: Update a test case.
//
//   - name: Identifier. The name of the test case resource. Format:
//     `projects/{project_number}/apps/{app}/testCases/{test_case}`.
func (r *ProjectsAppsTestCasesService) Patch(name string, googlefirebaseappdistrov1alphatestcase *GoogleFirebaseAppdistroV1alphaTestCase) *ProjectsAppsTestCasesPatchCall {
	c := &ProjectsAppsTestCasesPatchCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.name = name
	c.googlefirebaseappdistrov1alphatestcase = googlefirebaseappdistrov1alphatestcase
	return c
}

// AllowMissing sets the optional parameter "allowMissing": If set to true, and
// the test case is not found, a new test case will be created.
func (c *ProjectsAppsTestCasesPatchCall) AllowMissing(allowMissing bool) *ProjectsAppsTestCasesPatchCall {
	c.urlParams_.Set("allowMissing", fmt.Sprint(allowMissing))
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsAppsTestCasesPatchCall) Fields(s ...googleapi.Field) *ProjectsAppsTestCasesPatchCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsAppsTestCasesPatchCall) Context(ctx context.Context) *ProjectsAppsTestCasesPatchCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsAppsTestCasesPatchCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsAppsTestCasesPatchCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.googlefirebaseappdistrov1alphatestcase)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+name}")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("PATCH", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"name": c.name,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.patch", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.apps.testCases.patch" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaTestCase.ServerResponse.Header or (if a
// response was returned at all) in error.(*googleapi.Error).Header. Use
// googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsAppsTestCasesPatchCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaTestCase, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaTestCase{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.apps.testCases.patch", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type ProjectsTestersGetUdidsCall struct {
	s            *Service
	project      string
	urlParams_   gensupport.URLParams
	ifNoneMatch_ string
	ctx_         context.Context
	header_      http.Header
}

// GetUdids: Get UDIDs of tester iOS devices in a project
//
//   - project: The name of the project, which is the parent of testers Format:
//     `projects/{project_number}`.
func (r *ProjectsTestersService) GetUdids(project string) *ProjectsTestersGetUdidsCall {
	c := &ProjectsTestersGetUdidsCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.project = project
	return c
}

// MobilesdkAppId sets the optional parameter "mobilesdkAppId": Unique id for a
// Firebase app of the format:
// {version}:{project_number}:{platform}:{hash(bundle_id)} Example:
// 1:581234567376:android:aa0a3c7b135e90289
func (c *ProjectsTestersGetUdidsCall) MobilesdkAppId(mobilesdkAppId string) *ProjectsTestersGetUdidsCall {
	c.urlParams_.Set("mobilesdkAppId", mobilesdkAppId)
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *ProjectsTestersGetUdidsCall) Fields(s ...googleapi.Field) *ProjectsTestersGetUdidsCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// IfNoneMatch sets an optional parameter which makes the operation fail if the
// object's ETag matches the given value. This is useful for getting updates
// only after the object has changed since the last request.
func (c *ProjectsTestersGetUdidsCall) IfNoneMatch(entityTag string) *ProjectsTestersGetUdidsCall {
	c.ifNoneMatch_ = entityTag
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *ProjectsTestersGetUdidsCall) Context(ctx context.Context) *ProjectsTestersGetUdidsCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *ProjectsTestersGetUdidsCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *ProjectsTestersGetUdidsCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "", c.header_)
	if c.ifNoneMatch_ != "" {
		reqHeaders.Set("If-None-Match", c.ifNoneMatch_)
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1alpha/{+project}/testers:udids")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("GET", urls, nil)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	googleapi.Expand(req.URL, map[string]string{
		"project": c.project,
	})
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.testers.getUdids", "request", internallog.HTTPRequest(req, nil))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "firebaseappdistribution.projects.testers.getUdids" call.
// Any non-2xx status code is an error. Response headers are in either
// *GoogleFirebaseAppdistroV1alphaGetTesterUdidsResponse.ServerResponse.Header
// or (if a response was returned at all) in error.(*googleapi.Error).Header.
// Use googleapi.IsNotModified to check whether the returned error was because
// http.StatusNotModified was returned.
func (c *ProjectsTestersGetUdidsCall) Do(opts ...googleapi.CallOption) (*GoogleFirebaseAppdistroV1alphaGetTesterUdidsResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &GoogleFirebaseAppdistroV1alphaGetTesterUdidsResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "firebaseappdistribution.projects.testers.getUdids", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}
